/************************************************************/
/*                                                          */
/*             CONFIGURATION LIBRARY HEADER FILE            */
/*                                                          */
/************************************************************/
/*                                                          */
/*      MODULE     : CfgParse.h                             */
/*      AUTHOR     : F. BELLACHIA                           */
/*      LABORATORY : L.A.P.P.                               */
/*      VERSION    : v0r0                                   */
/*      DATE       : 09/03/00                               */
/*                                                          */
/************************************************************/
/*                                                          */
/*      MODIFICATION:                                       */
/*                                                          */
/************************************************************/
/*                                                          */
/*                                                          */
/************************************************************/
/* Copyright (c) 1999 LAPP Annecy, UMR-IN2P3-CNRS (France). */
/*                                                          */
/* Redistribution and use in source and binary forms, with  */
/* or without modification, are permitted provided that the */
/* following conditions are met:                            */
/* 1. Redistributions of source code must retain the above  */
/*    copyright notice, this list of conditions and the     */
/*    following disclaimer.                                 */
/* 2. Redistributions in binary form must reproduce the     */
/*    above copyright notice, this list of conditions and   */
/*    the following disclaimer in the documentation and/or  */
/*    other materials provided with the distribution.       */
/* 3. All advertising materials mentioning features or use  */
/*    of this software must display the following           */
/*    acknowledgement:                                      */
/*      This product includes software developed by the     */
/*      Computer Application Development Group at LAPP      */
/*      Annecy  (Laboratoire d'Annecy le vieux de Physique  */
/*      des Particules - UMR-IN2P3-CNRS).                   */
/* 4. Neither the name of the Institute nor of the          */
/*    Laboratory may be used to endorse or promote products */
/*    derived from this software without specific prior     */
/*    written permission.                                   */
/*                                                          */
/* THIS SOFTWARE IS PROVIDED BY THE LAPP AND CONTRIBUTORS   */
/* ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES,         */
/* INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF */
/* MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE */
/* DISCLAIMED. IN NO EVENT SHALL THE LAPP OR CONTRIBUTORS   */
/* BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, */
/* EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT  */
/* LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; */
/* LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) */
/* HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER   */
/* IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING        */
/* NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE   */
/* USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY */
/* OF SUCH DAMAGE.                                          */
/************************************************************/

#include <CfgHeader.h>

#ifndef CfgParse_h
#define CfgParse_h

#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */

#define CFG_NO_ARG        -3
#define CFG_KEY_ATTRIBUTE 0xFACEBABE
#define CFG_KEY           0xBADEDEAD
#define CFG_NONE_KEY      0xFFFFFFFF
#define CFG_MAX_ATTR      128 
#define CFG_MAX_STACK     CFG_MAX_ATTR + 4
#define CFG_MAX_HENUM     256           
#define CFG_DSIZE         (sizeof(double)/sizeof(Ulong))
#define CFG_MAX_NAME      256
#define CFG_MAXPATHLEN    1024

#define CFG_VA_LIST_IDX(idx, narg) -(((idx) << 16) | ((narg) & 0xFFFF))
#define CFG_VA_LIST                CFG_VA_LIST_IDX(1, 1)

typedef enum CfgType{
  CfgNone,                                            /* Dummy entry not used */
  CfgFlag,                                         /* No attribute just a key */
  CfgConstString,                                  /* Static string attribute */
  CfgString,                                              /* String attribute */
  CfgBoolean,                                            /* Boolean attribute */
  CfgReal,                                                /* Double attribute */
  CfgHex,                                           /* Long integer attribute */
  CfgDec,                                                /* Integer attribute */
  CfgFunction,                                          /* Function attribute */
  CfgEnum,                                                  /* Enum attribute */
  CfgProcessing,                                      /* Processing attribute */
  CfgGetFunction			       /* No stack function attribute */
} CfgType_t;

typedef union CfgDouble{
  Ulong  i[2];
  double d;
} CfgDouble_t;

typedef struct CfgParse CfgParse_t;
typedef struct CfgDataType CfgDataType_t;

extern int   CfgBufToFile(char *, char *);
extern char  *CfgDbGet(int, void *, void *(*)(), char *(*)());
extern int   CfgDbPut(char *, char *, void (*)());
extern char  *CfgFileToBuf(char *);
extern char  *CfgMapFile(char *);
extern int   CfgParseAdd(CfgParse_t *, char *, int, ...);
extern int   CfgParseArg(CfgParse_t *, int, char **);
extern int   CfgParseBuffer(CfgParse_t *, char *);
extern char  *CfgParseExtract(CfgParse_t *, char *, CfgBool_t);
extern int   CfgParseFile(CfgParse_t *, char *);
extern CfgParse_t *CfgParseFree(CfgParse_t *);
extern int   CfgParseFunctionAdd    (CfgParse_t *, char *, int (*)(), void *,
				     int, ...);
extern int   CfgParseGetFunctionAdd (CfgParse_t *, char *, 
				     int (*)(void *, CfgDataType_t *), 
				     void *,
				     int, ...);

extern double    CfgParseGetNextReal 	    (CfgDataType_t *dataType);
extern long int  CfgParseGetNextHex 	    (CfgDataType_t *dataType);
extern int   	 CfgParseGetNextDec 	    (CfgDataType_t *dataType);
extern char 	*CfgParseGetNextString 	    (CfgDataType_t *dataType);
char 		*CfgParseGetNextConstString (CfgDataType_t *dataType);
extern int   	 CfgParseGetNextEnum 	    (CfgDataType_t *dataType);

int 		 CfgParseGetResult 	    (CfgDataType_t *dataType);
  
extern CfgParse_t *CfgParseNew(void);
extern int   CfgParseSetEnumAttr(CfgParse_t *, char *, long);
extern char  *CfgPrintError(void);
extern int   CfgSetErrorMsg(char *, ...);

#ifdef __cplusplus
}
#endif /* __cplusplus */

#endif /* CfgParse_h */
