/************************************************************/
/*                                                          */
/*                    MANAGE QUEUE HEADER FILE              */
/*                                                          */
/************************************************************/
/*                                                          */
/*      MODULE     : CfgQueue.h                             */
/*      AUTHOR     : F. BELLACHIA                           */
/*      LABORATORY : L.A.P.P.                               */
/*      VERSION    : v0r0                                   */
/*      DATE       : 29/03/00                               */
/*                                                          */
/************************************************************/
/*                                                          */
/*      MODIFICATION:                                       */
/*                                                          */
/************************************************************/
/*                                                          */
/*                                                          */
/************************************************************/
/* Copyright (c) 2000 LAPP Annecy, UMR-IN2P3-CNRS (France). */
/*                                                          */
/* Redistribution and use in source and binary forms, with  */
/* or without modification, are permitted provided that the */
/* following conditions are met:                            */
/* 1. Redistributions of source code must retain the above  */
/*    copyright notice, this list of conditions and the     */
/*    following disclaimer.                                 */
/* 2. Redistributions in binary form must reproduce the     */
/*    above copyright notice, this list of conditions and   */
/*    the following disclaimer in the documentation and/or  */
/*    other materials provided with the distribution.       */
/* 3. All advertising materials mentioning features or use  */
/*    of this software must display the following           */
/*    acknowledgement:                                      */
/*      This product includes software developed by the     */
/*      Computer Application Development Group at LAPP      */
/*      Annecy  (Laboratoire d'Annecy le vieux de Physique  */
/*      des Particules - UMR-IN2P3-CNRS).                   */
/* 4. Neither the name of the Institute nor of the          */
/*    Laboratory may be used to endorse or promote products */
/*    derived from this software without specific prior     */
/*    written permission.                                   */
/*                                                          */
/* THIS SOFTWARE IS PROVIDED BY THE LAPP AND CONTRIBUTORS   */
/* ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES,         */
/* INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF */
/* MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE */
/* DISCLAIMED. IN NO EVENT SHALL THE LAPP OR CONTRIBUTORS   */
/* BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, */
/* EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT  */
/* LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; */
/* LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) */
/* HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER   */
/* IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING        */
/* NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE   */
/* USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY */
/* OF SUCH DAMAGE.                                          */
/************************************************************/

//#include <time.h> 
#include <semaphore.h>
//#include <pthread.h>
//#include <stdarg.h>
#include <CfgHeader.h>
#ifdef USE_MW
#include <memwatch.h>
#endif /* USE_MW */

#ifndef CfgQueue_h
#define CfgQueue_h

#define CFGQUEUE_CHECK -1.

typedef struct CfgRequest{
  void              *request;                           /* Pointer of request */
  struct CfgRequest *next;           /* Pointer to next request, NULL if none */
} CfgRequest_t;

typedef int (*CfgQueuePrinter_t)(char* format, va_list args);

typedef struct CfgQueue{
  char            *name;                                      /* Queue's name */
  pthread_mutex_t lock;                                      /* Queue's mutex */
  sem_t           sync;                                  /* Queue's semaphore */
  int             nbRequest;                    /* Number of request in queue */
  int             threshold;                            /* Threshold of queue */
  CfgRequest_t    *head;                    /* Head of linked-list of request */
  CfgRequest_t    *tail;                    /* Tail of linked-list of request */
}CfgQueue_t;

extern void              *CfgDequeue (CfgQueue_t *queue, double timeout);
extern int               CfgEnqueue (CfgQueue_t *queue, void *request);
extern int               CfgQueueFlush (CfgQueue_t *queue, int nbRequest);
extern CfgQueue_t        *CfgQueueFree (CfgQueue_t *queue);
extern int               CfgQueueGetValue (CfgQueue_t *queue);
extern CfgQueuePrinter_t CfgQueueInstallPrinter (CfgQueuePrinter_t printer);
extern CfgQueue_t        *CfgQueueNamedNew(int nelem, char *name);
extern CfgQueue_t        *CfgQueueNew (int nelem);

#endif /* CfgQueue_h */
